'use strict';

var walkNodes = require('jsonschema-nodewalker');

/**
 *
 */
module.exports = function (types) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var _options$titleDelimit = options.titleDelimiter,
      titleDelimiter = _options$titleDelimit === undefined ? '-' : _options$titleDelimit;


  var TYPE_MAP = Object.freeze({
    boolean: function boolean(node, meta) {
      return types.boolean;
    },
    number: function number(node, meta) {
      return types.number;
    },
    string: function string(node, meta) {
      var format = node.format;
      if (format === 'datetime') return types.Date;
      return types.string;
    },
    object: function object(node, meta) {
      var title = node.title && formatTitle(node.title, { titleDelimiter: titleDelimiter });
      return Object.keys(meta.childObjectProperties).length ? title ? types.model(title, meta.childObjectProperties) : types.model(meta.childObjectProperties) : types.frozen;
    },
    array: function array(node, meta) {
      return types.array(meta.childArrayItem);
    }
  });

  return function () {
    var schema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var onNode = arguments[1];
    return walkNodes(schema, function (node, meta) {
      var type = TYPE_MAP[node.type](node, meta);
      var hasDefault = node.default !== undefined;
      var isRequired = meta.isRequired || !meta.lineage;
      var result = !isRequired || hasDefault ? hasDefault ? types.optional(type, node.default) : types.maybe(type) : type;
      return onNode ? onNode(result, { node: node, meta: meta, typeMap: TYPE_MAP }) : result;
    });
  };
};

/**
 *
 */
function formatTitle(title, _ref) {
  var titleDelimiter = _ref.titleDelimiter;

  return title.replace ? title.replace(/\s+/g, titleDelimiter) : title;
}